/***************************************
	Auteur : Pierre Aubert
	Mail : aubertp7@gmail.com
	Licence : CeCILL-C
****************************************/

/*{
First, we have to activate the \textbf{numpy} we disabled in the previous files.
Otherwise, you will get a segmentation fault out of knowhere.
}*/

#define NPY_NO_DEPRECATED_API NPY_1_7_API_VERSION
#ifndef DISABLE_COOL_ARRAY
#define PY_ARRAY_UNIQUE_SYMBOL core_ARRAY_API
#endif

/*{
Then, we include all the previous wrapper and the python stuff, of course.
}*/

#include <iostream>

#include <Python.h>
#include "structmember.h"
#include <numpy/arrayobject.h>

#include "hadamardWrapper.h"

/*{
It is always good to have a documentation of your function. The folowing will appear when you call a function with '?' or shift-tab in \textbf{jupyter-notebook} :
}*/

std::string hadamardWrapper_docstring = "Compute a Hadamard product with aligned table of float32\n\
Parameters :\n\
	tabRes : table of the results (float32 aligned)\n\
	tabX : table of value (float32 aligned)\n\
	tabY : table of value (float32 aligned)\n\
Return :\n\
	Result of the Hadamard as numpy array (tabRes)";

/*{
We define the callable method of our wrapper :
}*/

static PyMethodDef _hadamardpython_methods[] = {
	{"hadamard", (PyCFunction)hadamardWrapper, METH_VARARGS, hadamardWrapper_docstring.c_str()},

	{NULL, NULL}
};

/*{
\begin{advise}
	Notice that you can expose the functions you want, not all the functions you developed.
\end{advise}

Now, we have to define the \textbf{python} module itself :
}*/

static PyModuleDef _hadamardpython_module = {
	PyModuleDef_HEAD_INIT,
	"hadamardpython",
	"",
	-1,
	_hadamardpython_methods,
	NULL,
	NULL,
	NULL,
	NULL
};

/*{
Now we define the function which will be called on the \textbf{import} of our module.

\begin{warning}
	Notice you must call this function \textbf{PyInit_}NameOfYouModule where \textbf{NameOfYouModule} is the name of \textbf{THIS} file.
\end{warning}

We create the module, which is a \textbf{PyObject}.

\begin{warning}
	Do not forget to call \textbf{import_array} otherwise you will get a warning on the best cases and a segmentation fault on your module \textbf{import}.
\end{warning}
}*/

///Create the python module hadamardpython
/**	@return python module hadamardpython
*/
PyMODINIT_FUNC PyInit_hadamardpython(void){
	PyObject *m;
	import_array();
	
	m = PyModule_Create(&_hadamardpython_module);
	if(m == NULL){
		return NULL;
	}
	return m;
}


